const express = require('express');

const enrollmentApp = express.Router();

const libs = `${process.cwd()}/libs`;

const log = require(`${libs}/log`)(module);

const squel = require('squel');

const query = require(`${libs}/middlewares/query`);

const response = require(`${libs}/middlewares/response`);

const parseParams = require(`${libs}/middlewares/parseParams`);

// **Temporary** solution to add where clauses that are common to all requests


// Complete range of the enrollments dataset.
// Returns a tuple of start and ending years of the complete enrollments dataset.
enrollmentApp.get('/year_range', (req, res, next) => {
    req.sql = squel.select()
        .from('turmas')
        .field('MIN(turmas.ano_censo)', 'start_year')
        .field('MAX(turmas.ano_censo)', 'end_year');

    next();
}, query, response('range'));

// Returns all educational levels avaible
enrollmentApp.get('/education_level', (req, res, next) => {
    req.sql = squel.select()
        .from('etapa_ensino')
        .field('pk_etapa_ensino_id', 'id')
        .field('desc_etapa', 'name');

    next();
}, query, response('education_level'));

// Returns all adm dependencies
enrollmentApp.get('/adm_dependency', (req, res, next) => {
    req.sql = squel.select()
        .from('dependencia_adms')
        .field('pk_dependencia_adm_id', 'id')
        .field('nome', 'name');

    next();
}, query, response('adm_dependency'));

// Parse the filters and dimensions parameter in the query
enrollmentApp.use('/', parseParams('filter', [
    'min_year',
    'max_year',
    'adm_dependency_id',
    'location_id',
    'education_level_id',
    'region',
    'state',
    'city',
    'school'
]), parseParams('dims', [
    'adm_dependency_id',
    'location_id',
    'education_level_id',
    'region',
    'state',
    'city',
    'school'
]), (req, res, next) => {
    log.debug(req.filter);
    log.debug(req.dims);

    // Do the joins
    if(typeof req.filter.adm_dependency_id !== 'undefined'
        || typeof req.dims.adm_dependency_id !== 'undefined') {
        req.sql.join('dependencia_adms', null, 'fk_dependencia_adm_id=dependencia_adms.pk_dependencia_adm_id');
    }

    if(typeof req.filter.education_level_id !== 'undefined'
        || typeof req.dims.education_level_id !== 'undefined') {
        req.sql.join('etapa_ensino', null, 'fk_etapa_ensino_id=etapa_ensino.pk_etapa_ensino_id');
    }

    if(typeof req.filter.region !== 'undefined'
        || typeof req.dims.region !== 'undefined') {
            req.sql.join('municipios', null, 'fk_municipio_id=municipios.pk_municipio_id')
                .join('estados', null, 'municipios.fk_estado_id=estados.pk_estado_id')
                .join('regioes', null, 'estados.fk_regiao_id=regioes.pk_regiao_id');
    }

    if((typeof req.filter.state !== 'undefined'
        || typeof req.dims.state !== 'undefined')
        && (typeof req.filter.region === 'undefined'
        && typeof req.dims.region === 'undefined')) {
            req.sql.join('municipios', null, 'fk_municipio_id=municipios.pk_municipio_id')
                .join('estados', null, 'municipios.fk_estado_id=estados.pk_estado_id');
    }

    if((typeof req.filter.city !== 'undefined'
        || typeof req.dims.city !== 'undefined')
        && (typeof req.filter.state === 'undefined'
        && typeof req.dims.state === 'undefined')
        && (typeof req.filter.region === 'undefined'
        && typeof req.dims.region === 'undefined')) {
        req.sql.join('municipios', null, 'fk_municipio_id=municipios.pk_municipio_id');
    }

    if(typeof req.dims.school !== 'undefined') {
        req.sql.join('escolas', null, 'fk_escola_id=escolas.pk_escola_id');
    }

    // Dimensions (add fields)

    if(typeof req.dims.education_level_id !== 'undefined') {
        req.sql.field('desc_etapa', 'education_level')
            .group('desc_etapa')
            .order('desc_etapa');
    }

    if(typeof req.dims.region !== 'undefined') {
        req.sql.field('regioes.nome', 'region_name')
            .group('regioes.nome')
            .order('regioes.nome');
    }

    if(typeof req.dims.state !== 'undefined') {
        req.sql.field('estados.nome', 'state_name')
            .group('estados.nome')
            .order('estados.nome');
    }

    if(typeof req.dims.city !== 'undefined') {
        req.sql.field('municipios.nome', 'city_name')
            .group('municipios.nome')
            .order('municipios.nome');
    }

    if(typeof req.dims.school !== 'undefined') {
        req.sql.field('escolas.nome_entidade', 'school_name')
            .group('escolas.nome_entidade')
            .order('escolas.nome_entidade');
    }

    if(typeof req.dims.adm_dependency_id !== 'undefined') {
        req.sql.field('dependencia_adms.nome', 'adm_dependency_name')
            .group('dependencia_adms.nome')
            .order('dependencia_adms.nome');
    }

    if(typeof req.dims.location_id !== 'undefined') {
        req.sql.field('turmas.id_localizacao', 'location')
            .group('turmas.id_localizacao')
            .order('turmas.id_localizacao');
    }

    if(typeof req.dims.region === 'undefined'
        && typeof req.dims.state === 'undefined'
        && typeof req.dims.city === 'undefined'
        && typeof req.dims.school === 'undefined') {
        req.sql.field("'Brasil'", 'name');
    }

    // Filter (add where)

    if (typeof req.filter.min_year !== 'undefined') {
        req.sql.where('turmas.ano_censo>=?', parseInt(req.filter.min_year, 10));
    }

    if (typeof req.filter.max_year !== 'undefined') {
        req.sql.where('turmas.ano_censo<=?', parseInt(req.filter.max_year, 10));
    }

    if (typeof req.filter.adm_dependency_id !== 'undefined') {
        req.sql.where('pk_dependencia_adm_id=?', parseInt(req.filter.adm_dependency_id, 10));
    }

    if (typeof req.filter.location_id !== 'undefined') {
        req.sql.where('turmas.id_localizacao=?', parseInt(req.filter.location_id, 10));
    }

    if (typeof req.filter.education_level_id !== 'undefined') {
        req.sql.where('pk_etapa_ensino_id=?', parseInt(req.filter.education_level_id, 10));
    }

    if (typeof req.filter.region !== 'undefined') {
        req.sql.where('pk_regiao_id=?', parseInt(req.filter.region, 10));
    }

    if (typeof req.filter.state !== 'undefined') {
        req.sql.where('pk_estado_id=?', parseInt(req.filter.state, 10));
    }

    if (typeof req.filter.city !== 'undefined') {
        req.sql.where('turmas.fk_municipio_id=?', parseInt(req.filter.city, 10));
    }

    if (typeof req.filter.school !== 'undefined') {
        req.sql.where('turmas.fk_escola_id=?', parseInt(req.filter.school, 10));
    }
    log.debug(req.sql.toParam());
    next();
});

enrollmentApp.get('/', (req, res, next) => {
    req.sql.field('COALESCE(SUM(num_matriculas), 0)', 'total')
        .field('turmas.ano_censo', 'year')
        .from('turmas')
        .group('turmas.ano_censo')
        .order('turmas.ano_censo');
    next();
}, query, (req, res, next) => {
    if(req.result.length === 0) {
        req.result.push({total: 0});
    }
    next();
}, response('enrollment'));

module.exports = enrollmentApp;
